# frozen_string_literal: true

module Hermes
  module Tools
    # Tools for specs
    module Specs
      def get_typed_value(type, iteration = 0)
        return type[iteration] if type.kind_of?(Array)
        type_array = case type
                     when 'String'
                       %w[alpha bravo charlie]
                     when 'Boolean'
                       [false, true, true]
                     when 'Hash'
                       [
                           {'alpha' => 'bravo'},
                           {'charlie' => 'delta'},
                           {'echo' => 'foxtrott'}
                       ]
                     when '[String]'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array[Coord]'
                       [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]
                     when 'Date'
                       [
                           [2002, 2, 3, 5],
                           [2003, 2, 3, 8],
                           [2004, 2, 3, 6]
                       ].map {|arr| Date.new(*arr)}
                     when 'DateTime'
                       [
                           [2001, 2, 3, 4, 4, 4],
                           [2002, 2, 3, 4, 4, 6],
                           [2003, 2, 3, 4, 4, 6]
                       ].map {|arr| DateTime.new(*arr)}
                     when 'Integer'
                       [1, 2, 3]
                     when 'Float'
                       [1.1, 2.2, 3.3]
                     when 'Siren'
                      ['123456780', '223456789', '323456789']
                     when 'Siret'
                      ['12345678901234', '22345678901234', '32345678901234']
                     else
                       puts "ERROR: Unknow type #{type}"
                       [nil, nil, nil]
                     end
        type_array[iteration]
      end

      def nice_type(type)
        case type
        when 'String'
          nice_type = :string
        when 'Boolean'
          nice_type = :boolean
        when 'Hash'
          nice_type = :object
        when 'Array'
          nice_type = :array
        when 'Date'
          nice_type = :date
        when 'DateTime'
          nice_type = :time
        when 'Integer'
          nice_type = :integer
        when 'Float'
          nice_type = :float
        else
          puts 'ERROR: Unknow type'
        end
        nice_type
      end

      def nice_typev2(type, opt={})
        # see https://swagger.io/docs/specification/data-models/data-types for more options
        case type
        when 'String'
          return { type: :string, **opt }
        when 'Boolean'
          return { type: :boolean, **opt }
        when 'Hash'
          return { type: :object, **opt }
        when 'Array'
          return { type: :array, **opt }
        when 'Date'
          return { type: :string, format: :date, **opt }
        when 'DateTime'
          return { type: :string, format: :datetime, **opt }
        when 'Integer'
          return { type: :integer, **opt }
        when 'Float'
          return { type: :float, **opt }
        else
          puts 'ERROR: Unknow type'
        end
      end

      def jsonify_params(params, opt={})
        schema = {
            type: :object,
            properties: {
                data: {
                    type: :object,
                    properties: {
                        attributes: {
                            type: :object,
                            properties: {},
                            required: opt.fetch(:required, [])
                        }
                    }
                    #required: []
                }
            }
            #required: []
        }
        params.each do |key, val|
          if val[:type] == :array
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: :array, items: { type: :string } }
          else
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: val[:type]}
          end
        end
        schema
      end

      def check_mutable_params(opts)
        model = opts[:model] 
        mutate_params = opts[:mutate_params]
        factory = opts[:factory]
        check_post = opts.has_key?(:check_post) ? opts[:check_post] : true
        check_put = opts.has_key?(:check_put) ? opts[:check_put] : true
        min_params = opts.has_key?(:min_params) ? opts[:min_params] : {}
        embedded_params = opts.has_key?(:embedded_params) ? opts[:embedded_params] : {}
        other_factory = opts.has_key?(:other_factory) ? opts[:other_factory] : []
        user_roles =  opts.has_key?(:user_roles) ? opts[:user_roles] : []
        auth_roles =  opts.has_key?(:auth_roles) ? opts[:auth_roles] : ['superadministrateur']
        not_checked_keys = opts.has_key?(:not_checked_keys) ? opts[:not_checked_keys] : []

        not_mutable_params = model.attribute_names.map(&:to_sym) - mutate_params.keys - not_checked_keys
        show_json = false

        describe "#{model.class.name} check mutable" do 
          before do
            HERMES_CONFIG[:ignore_cassette] = true
          end

          let!(:responsible) {create(:user, auth_roles: auth_roles, roles: user_roles)}

          other_factory.each do |o|
            let!(:"#{o[:id]}") {create(o[:factory], (o[:params] || {}).merge(id: o[:id]))}
          end 

          if check_post
            describe 'post' do 
              mutate_params.each do |param, type|
                describe "with #{param.to_s} (#{type.to_s}) is valid" do
                  let(:params) { {"#{param}": get_typed_value(type)} }
                  it 'returns status code 201 and the value appears in json' do
                    authenticated_header(request, responsible)
                    post 'create', {data: { attributes: min_params.merge(params) }}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(201)
                    if (type == 'Date' || type == 'DateTime' || type.kind_of?(Array))
                      expect(json['data']['attributes'][param.to_s]).not_to eq('')
                    else
                      expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
                    end
                  end
                end
              end
  
              not_mutable_params.each do |param|
                describe "with #{param.to_s} is not mutable"  do
                  it 'returns status 422 and show error' do
                    authenticated_header(request, responsible)
                    post 'create', { data: { attributes: min_params.merge({"#{param}": "test"})}}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(422)
                    expect(json["errors"].first.symbolize_keys).to eq({"#{param}": "not mutable"})
                  end
                end
              end 
            end
          end 

          if check_put 
            describe 'put' do 
              let!(:default_model) {create(factory, min_params)}

              mutate_params.each do |param, type|
                describe "with #{param.to_s} (#{type.to_s}) is valid" do
                  let(:params) { {"#{param}": get_typed_value(type, 1)} }
                  it 'returns status code 200 and the value appears in json' do
                    authenticated_header(request, responsible)
                    put 'update', {id: default_model.id, data: { attributes: params }}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(200)
                    if (type == 'Date' || type == 'DateTime' || type.kind_of?(Array))
                      expect(json['data']['attributes'][param.to_s]).not_to eq('')
                    else
                      expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 1))
                    end
                  end
                end
              end
    
              not_mutable_params.each do |param|
                describe "with #{param.to_s} is not mutable"  do
                  it 'returns status 422 and show error' do
                    authenticated_header(request, responsible)
                    put 'update', { id: default_model.id.to_s, data: { attributes: {"#{param}": "test"}}}.merge(embedded_params)
                    ap json if show_json
                    expect(response).to have_http_status(422)
                    expect(json["errors"].first.symbolize_keys).to eq({"#{param}": "not mutable"})
                  end
                end
              end 
            end 
          end 
          after do
            HERMES_CONFIG[:ignore_cassette] = false
          end
        end 
      end 

    end
  end
end

include Hermes::Tools::Specs
